/*
This source file is part of the RGBZ Video Camera project published in the paper

  Coherent Spatiotemporal Filtering, Upsampling and Rendering of RGBZ Videos
  Christian Richardt, Carsten Stoll, Neil A. Dodgson, Hans-Peter Seidel, Christian Theobalt
  Computer Graphics Forum (Proceedings of Eurographics), 31(2), May 2012
  <http://www.mpi-inf.mpg.de/resources/rgbz-camera/>

If you use our code, please cite our paper. Thanks.

Copyright (c) 2012 Christian Richardt <christian@richardt.name>


Code licence: GPL v3
----------------------------------------------------------------------------------------------------
This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, see <http://www.gnu.org/licenses/>.

$Id: GaussianY.frag 537 2012-04-04 15:35:42Z cr333 $
*/
#version 120
#extension GL_EXT_gpu_shader4 : enable

//---- Inputs -------------------------------------------------------------------------------------

// input texture
uniform sampler2D src;

// standard deviation of the Gaussian
uniform float sigma;

//---- Main ---------------------------------------------------------------------------------------
	
void main()
{
	// find out size of texture/viewport to address individual pixels
	vec2 viewport = textureSize2D(src, 0);
	
	// look up current pixel
	vec3 centre = texture2D(src, gl_FragCoord.xy / viewport).rgb;
	
	// use a homogeneous accumulator for pixel values, with weights in alpha channel
	vec4 acc = vec4(centre, 1.0f); // init with centre pixel
	
	// precompute this constant
	float sigma2 = 1.0f / (2.0f * sigma * sigma);
	
	// work through pixels at equal distances from centre
	for(float dy = 1.0f; dy <= 2.0f * sigma; dy++)
	{
		// look up their colour values
		vec3 col1 = texture2D(src, (gl_FragCoord.xy + vec2(0.0f, dy)) / viewport).rgb;
		vec3 col2 = texture2D(src, (gl_FragCoord.xy - vec2(0.0f, dy)) / viewport).rgb;
		
		// add both colours to accumulator, with given weight
		acc += exp(-dy * dy * sigma2) * vec4(col1 + col2, 2);
	}

	// normalise by sum of weights in alpha channel
	gl_FragColor = vec4(acc.rgb / acc.a, 1);
}
