﻿// $Id: BilateralFilter.cs 532 2011-05-02 16:45:13Z cr333 $
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

namespace VideoLib.OpenGL
{
	/// <summary>Applies a bilateral filter.</summary>
	public class BilateralFilter : FboNode
	{
		/// <summary>Gets or sets the spatial sigma.</summary>
		[Uniform(), Description("Standard deviation for spatial filtering.")]
		public float SpatialSigma { get; set; }

		/// <summary>Gets or sets the range sigma.</summary>
		[Uniform(), Description("Standard deviation for range filtering.")]
		public float RangeSigma { get; set; }

		/// <summary>Initialises a new instance of the <see cref="BilateralFilter"/> class.</summary>
		public BilateralFilter()
		{
			// default settings
			SpatialSigma = 4;
			RangeSigma   = 0.1f;

			AddDefaultPins();
		}

		/// <summary>Initialises the OpenGL state.</summary>
		public override void Init()
		{
			// load, compile and link shaders
			programID = Utils.LinkShaders(
				Utils.CreateShaderFromResource("Shaders.BilateralFilter.frag")
			);

			// map textures
			textureMap = new List<TextureBinding> {
				new TextureBinding { Pin = "in", Uniform = "input", Unit = 0 },
			};

			base.Init();
		}

		/// <summary>Draws to the framebuffer object using the given input textures.</summary>
		/// <param name="inputs">The input textures.</param>
		/// <returns>The resulting texture.</returns>
		public override TextureFrame Draw(Dictionary<string, TextureFrame> inputs)
		{
			// allocate a texture that is identical to the input
			var resultTexture = TextureFrame.CreateToMatch(inputs["in"]);
			resultTexture.Alloc();

			Draw(inputs, resultTexture.Texture);

			return resultTexture;
		}
	}
}
