﻿// $Id: StereoVideoControl.frag 557 2012-04-18 12:50:25Z cr333 $
#extension GL_EXT_gpu_shader4 : enable

// Input textures of the two stereo views.
uniform sampler2D inputL;
uniform sampler2D inputR;

// Stereo display mode.
uniform int stereoMode;
const int LeftOnly          = 1 << 1;
const int RightOnly         = 1 << 2;
const int SwapViews         = 1 << 3;
const int Overlay           = 1 << 4;
const int RowInterlacedEven = 1 << 5;

const int LeftRed      = 1 << 13;
const int LeftGreen    = 1 << 14;
const int LeftBlue     = 1 << 15;
const int RightRed     = 1 << 10;
const int RightGreen   = 1 << 11;
const int RightBlue    = 1 << 12;

const int GreyAnaglyph       = 1 << 20;
const int FullColourAnaglyph = 1 << 21;

// Screen coordinate of the top-left corner of control, e.g. used for row interlacing.
uniform vec2 screenPos;

void main()
{
	// Look up this pixel's colour in both stereo views.
	vec4 colourL = texture2D(inputL, gl_TexCoord[0].xy);
	vec4 colourR = texture2D(inputR, gl_TexCoord[0].xy);

	// Swap the left and right views.
	if((stereoMode & SwapViews) > 0)
	{
		vec4 oldLeft = colourL;
		colourL = colourR;
		colourR = oldLeft;
	}

	if((stereoMode & LeftOnly) > 0)
	{
		gl_FragColor = vec4(1.0, 0.0, 0.0, 1.0);
	}
	else if((stereoMode & RightOnly) > 0)
	{
		gl_FragColor = vec4(1.0, 1.0, 0.0, 1.0);
	}
	else if((stereoMode & Overlay) > 0)
	{
		gl_FragColor = 0.5 * colourL + 0.5 * colourR;
	}
	else if((stereoMode & FullColourAnaglyph) > 0)
	{
		// First clear output colour.
		gl_FragColor = vec4(0.0, 0.0, 0.0, 1.0);

		// Copy colour components from left image.
		if((stereoMode & LeftRed)    > 0) gl_FragColor.r = colourL.r;
		if((stereoMode & LeftGreen)  > 0) gl_FragColor.g = colourL.g;
		if((stereoMode & LeftBlue)   > 0) gl_FragColor.b = colourL.b;

		// Copy colour components from right image.
		if((stereoMode & RightRed)   > 0) gl_FragColor.r = colourR.r;
		if((stereoMode & RightGreen) > 0) gl_FragColor.g = colourR.g;
		if((stereoMode & RightBlue)  > 0) gl_FragColor.b = colourR.b;
	}
	else if((stereoMode & GreyAnaglyph) > 0)
	{
		// First clear output colour.
		gl_FragColor = vec4(0.0, 0.0, 0.0, 1.0);
		
		// Compute pixel intensities.
		float greyL = dot(vec4(0.2126729, 0.7151522, 0.0721750, 0.0), colourL);
		float greyR = dot(vec4(0.2126729, 0.7151522, 0.0721750, 0.0), colourR);

		// Copy colour components from left image.
		if((stereoMode & LeftRed)    > 0) gl_FragColor.r = greyL;
		if((stereoMode & LeftGreen)  > 0) gl_FragColor.g = greyL;
		if((stereoMode & LeftBlue)   > 0) gl_FragColor.b = greyL;

		// Copy colour components from right image.
		if((stereoMode & RightRed)   > 0) gl_FragColor.r = greyR;
		if((stereoMode & RightGreen) > 0) gl_FragColor.g = greyR;
		if((stereoMode & RightBlue)  > 0) gl_FragColor.b = greyR;
	}
	else if((stereoMode & RowInterlacedEven) > 0)
	{
		// Compute row of this pixel.
		float row = screenPos.y + gl_FragCoord.y;

		if(int(mod(row, 2.0)) == 0) gl_FragColor = colourL;
		else                        gl_FragColor = colourR;
	}
	else
	{
		// Not implemented.
		gl_FragColor = vec4(0.0, 1.0, 0.0, 1.0);
	}
}