﻿// $Id: TextureOutput.cs 513 2010-12-17 12:47:41Z cr333 $
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using VideoLib.Frames;

namespace VideoLib.OpenGL
{
	/// <summary>Converter node that reads out a <see cref="TextureFrame"/> into a <see cref="BitmapFrame"/>.</summary>
	public class TextureOutput : Node, IOpenGLNode
	{
		/// <summary>Gets or sets the target pixel format.</summary>
		public PixelFormat TargetFormat { get; set; }

		/// <summary>Initialises a new instance of the <see cref="TextureOutput"/> class.</summary>
		public TextureOutput(PixelFormat targetFormat)
		{
			TargetFormat = targetFormat;

			AddInputPin("in");
			AddOutputPin("out");
		}

		/// <summary>Initialises a new instance of the <see cref="TextureOutput"/> class.</summary>
		public TextureOutput() : this(PixelFormat.BGR24)
		{
		}

		/// <summary>Processes some frames, to produce new frames.</summary>
		public override void Process()
		{
			// get the input texture frame
			Frame frame = GetInputFrame("in");
			if (PassOnEndOfStreamFrame(frame)) return;
			var textureFrame = CastFrameTo<TextureFrame>(frame);

			// read out the texture
			var result = textureFrame.Texture.ReadFrame(TargetFormat);
			result.SequenceNumber = textureFrame.SequenceNumber;
			textureFrame.RefDec();
			PushFrame(result);
		}

		/// <summary>Returns a <see cref="System.String"/> that represents this instance.</summary>
		/// <returns>A <see cref="System.String"/> that represents this instance.</returns>
		public override string ToString()
		{
			return String.Format("{0}({1})", Name, TargetFormat);
		}
	}
}
