﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace VideoLib.Parameters
{
    /// <summary>
    /// The exception that is thrown when a parameter is searched for in a single Type, but is not found.
    /// </summary>
    public class ParameterNotFoundException : Exception
    {
        private readonly string name;
        private readonly Type target;

        /// <summary>
        /// Creates a new instance of ParameterNotFoundException.
        /// </summary>
        /// <param name="searchedIn">The <see cref="Type"/> that was searched.</param>
        /// <param name="theName">The CLR name of the searched-for parameter (a parameter is a CLR property marked 
        /// with <see cref="ParameterAttribute"/>.</param>
        public ParameterNotFoundException(Type searchedIn, string theName)
        {
            target = searchedIn;
            name = theName;
        }

        /// <summary>
        /// The (CLR property) name of the parameter searched for.
        /// </summary>
        public string Name
        {
            get { return name; }
        }

        /// <summary>
        /// The type which was searched for the parameter.
        /// </summary>
        public Type Target
        {
            get { return target; }
        }

        /// <summary>
        /// Retrieves a string representation of the exception.
        /// </summary>
        /// <returns>A string describing the exception.</returns>
        public override string ToString()
        {
            return String.Format("Parameter {0} not found in type {1}", name, target.Name);
        }
    }
}
