﻿// $Id: NullVideoDisplay.cs 490 2010-11-09 18:51:34Z cr333 $
// Based on a Part II project by Rubin Xu (2010).
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using VideoLib.Frames;
using VideoLib.Parameters;

namespace VideoLib.Sinks
{
	/// <summary>A dummy VideoDisplay control that does not draw to screen.</summary>
	/// <remarks>
	/// Multiple <see cref="VideoDisplay"/>s will become interlocked when they call Graphics.Draw...() at high frequencies.
	/// Hence when benchmarking schedulers, it is better to use this dummy VideoDisplay instead.
	/// </remarks>
	public class NullVideoDisplay : Node
	{
		IVideoControl Target;

		/// <summary>Title of this display.</summary>
		protected string Title;

		/// <summary>
		/// Constructor, all arguments are dummy, they are here just to 
		/// make the signature same as VideoDisplay
		/// </summary>
		/// <param name="targetControl"></param>
		/// <param name="title"></param>
		public NullVideoDisplay(IVideoControl targetControl, string title)
			: base()
		{
			Target = targetControl;
			Title = title;
			AddInputPin("in");
		}

		/// <summary>Human-friendly node name.</summary>
		public override string Name
		{
			get { return String.Format("{0}({1})", base.Name, Title); }
		}

		/// <summary>
		/// Processes some frames, to produce new frames.
		/// </summary>
		public override void Process()
		{
			Frame frame = GetInputPin("in").GetFrame();
			// stop on end of stream
			if (frame is EndOfStreamFrame)
			{
				Status = NodeStatus.Finished;
				return;
			}
		}
	}
}
